var meso = meso || {};

meso.AbstractHighstockChart = (function() {

    //////////////// constructor
    var AbstractHighstockChart = function(options) {
        this.options = options;
        this._lazy = this._isLazy(options);
        this._dateTimeField = options.xAxis.field;
        // the series on each axis
        this.axisSeries = {};
        this.options.series.forEach(function(series, index) {
            var seriesAxisId = series.axis;
            if( !seriesAxisId ) {
                throw new Error("Series at index "+index+" has no 'axis' defined");
            }
            var axisSeriesList = this.axisSeries[seriesAxisId];
            if(!axisSeriesList) {
                axisSeriesList = [];
                this.axisSeries[seriesAxisId] = axisSeriesList;
            }
            axisSeriesList.push(series);
        }, this);
        this._fieldDefs = this._buildFieldDefs(options);
        // an array of the field names
        this._fields = this._fieldDefs.map(function(fieldDef) {
            return fieldDef.fieldId;
        }, this);

        this.chart = null;
        this._chartInitialized = false;
        this._chartData = {};
        this._create();
    }

    /////////////// public
    AbstractHighstockChart.prototype.destroy = function() {
        if(this.chart) {
            this.chart.destroy();
        }
    };

    /////////////// protected
    // abstract
    AbstractHighstockChart.prototype._fetchData = function(resultCallback, start, end) {
        // subclass must implement
        throw new Error("Subclass must implement abstract _fetchData method.");
    };
    // override to enable lazy mode
    AbstractHighstockChart.prototype._isLazy = function(options) {
        return false;
    };
    AbstractHighstockChart.prototype._buildFieldDefs = function(options) {
        var fieldDefs = [];
        // x-axis
        fieldDefs.push(new meso.FieldDef(
            options.xAxis.field,
            meso.Unit.ms, // unit must be ms
            0,
            meso.Agg.avg // TODO allow this to be configured?
        ));
        // y-axes
        options.yAxes.forEach(function(axis) {
            var axisUnit = axis.unit;
            // each seires on the axis
            this.axisSeries[axis.axisId].forEach(function(series) {
                fieldDefs.push(
                    meso.Util.applyDefaults({
                        unit: axisUnit // ignore the unit and use the axis unit, since they must match
                    }, series.fieldDef)
                );
            }, this);
        }, this);
        return fieldDefs;
    }
    AbstractHighstockChart.prototype._create = function() {
        //if( this._lazy ) {
            //this._createChart();
        //}
        this._loadInitialDataAndCreate();
    };
    AbstractHighstockChart.prototype._loadInitialDataAndCreate = function() {
        //this.chart.showLoading('Loading...');
        this._fetchData( meso.Util.bind(this, this._handleInitialLoad) );
    };
    AbstractHighstockChart.prototype._handleInitialLoad = function(data) {
        // if the chart data is lazily loaded (i.e. initially grouped), we
        // add this dummy row with current date in order to get the x-axis max 
        // to be the current date instead of the latest group date
        /*if( this._lazy ) {
            data.push((function() {
                var dummyRow = [];
                dummyRow.push(new Date().getTime());
                for( var i=1; i<data[0].length; i++ ) dummyRow[i] = null;
                return dummyRow;
            })());
        }*/
        //this.chart.hideLoading();
        this._loadChartData(data);
        this._createChart();
    };
    AbstractHighstockChart.prototype._loadChartData = function(data, append) {
        // initialize the data
        if( !append ) {
            this._fields.forEach(function(fieldId, index) {
                if( fieldId === this._dateTimeField ) return;
                this._chartData[fieldId] = [];
            }, this);
        }
        // load the data
        var row, dateTime, point, series;
        var dateTimeIndex = 0; // always 0
        data.forEach(function(row) {
            dateTime = row[dateTimeIndex];
            this._fields.forEach(function(fieldId, index) {
                if( index !== dateTimeIndex ) {
                    point = [dateTime, row[index]];
                    if( !append ) {
                        this._chartData[fieldId].push(point);
                    } else {
                        this.chart.get(fieldId).addPoint(point, 
                                false, // don't redraw
                                true,  // shift series data
                                false); // don't animate
                    }
                }
            }, this);
        }, this);
        // if the chart has been initialized then we need to reset the data on each series
        if( this._chartInitialized ) {
            this._fields.forEach(function(fieldId, index) {
                if( fieldId === this._dateTimeField ) return;
                this.chart.get(fieldId).setData(
                        this._chartData[fieldId], 
                        false); // false = don't redraw
            }, this);
        }
        // redraw the chart at the end if appending or if the data was replaced
        if( append || this._chartInitialized ) {
            this.chart.redraw();
        }
    }
    AbstractHighstockChart.prototype._createChart = function() {
        var chartOptions = this._buildChartOptions();
        this.chart = new Highcharts.StockChart(chartOptions);
    };
    AbstractHighstockChart.prototype._chartReady = function() {
        this._chartInitialized = true;
        /*if( this._lazy ) {
            this._loadInitialDataAndCreate();
        }*/
    };
    AbstractHighstockChart.prototype._onSetExtremes = function(e) {
        if( this._lazy ) {
            this._loadDataLazy(e.min, e.max);
        }
    };
    AbstractHighstockChart.prototype._loadDataLazy = function(min, max) {
        this.chart.showLoading('Loading...');
        var start, end;
        var currentExtremes = this.chart.xAxis[0].getExtremes();
        if( min !== currentExtremes.dataMin ) {
            start = Math.round(min / 1000);
        }
        if( max !== currentExtremes.dataMax ) {
            end = Math.round(max / 1000);
        }
        this._fetchData( meso.Util.bind(this, this._handleLazyLoad), start, end );
    };
    AbstractHighstockChart.prototype._handleLazyLoad = function(data) {
        this._loadChartData(data);
        this.chart.hideLoading();
    };
    AbstractHighstockChart.prototype._buildChartOptions = function() {
        var chartOptions = {};
        chartOptions.chart = {
            events: {
                load: meso.Util.bind(this, this._chartReady ),
            },
            alignTicks: true,
            animation: false
        };
        chartOptions.rangeSelector = {
            enabled: false
        };
        chartOptions.navigator = {
            enabled : true,
            adaptToUpdatedData : !this._lazy
        };
        chartOptions.plotOptions = {
            line: {
                lineWidth: 2,
                marker: {
                    enabled: false,
                    states: {
                        hover: {
                            enabled: true,
                            radius: 5
                        }
                    }
                },
                shadow: false,
                states: {
                    hover: {
                        lineWidth: 2
                    }
                }
            }
        };
        // x-axis
        var xAxisOptions = {
            type: 'datetime', // TODO is this necessary anymore?
            events: {
                setExtremes: meso.Util.bind(this, this._onSetExtremes)
            },
            ordinal: false,
        };
        chartOptions.xAxis = $.extend(true, xAxisOptions, this.options.xAxis.highstockAxisOptions);
        // y-axes & series
        chartOptions.yAxis = [];
        chartOptions.series = [];

        this.options.yAxes.forEach(function(axisDef, axisIndex) {
            
            var axisId = axisDef.axisId;
            if( !axisId ) {
                throw new Error("Axis at index "+axisIndex+" has no 'axisId' defined");
            }

            // create the highstock axis options
            var axis = $.extend(true, {}, axisDef.highstockAxisOptions);
            if( !axis.labels ) {
                axis.labels = {
                    formatter : this._suffixFormatter(axisDef.unit.labelSuffix)
                };
            }
            chartOptions.yAxis.push( axis );

            // create the highstock series options of this axis
            this.axisSeries[axisId].forEach(function(seriesDef) {
                var fieldDef = seriesDef.fieldDef;
                var fieldId = fieldDef.fieldId;
                var seriesDefaultOptions = {
                    name: fieldDef.label
                };
                // override the defaults
                var seriesOptions = $.extend(true, seriesDefaultOptions, seriesDef.highstockSeriesOptions);
                seriesOptions.id = fieldId;
                seriesOptions.yAxis = axisIndex;
                seriesOptions.data = this._chartData[fieldId];
                if( this._lazy ) {
                    // always disable data grouping if lazy
                    seriesOptions.dataGrouping = DATA_GROUPING_DISABLED;
                }
                seriesOptions.tooltip = {
                    valueDecimals: fieldDef.decimals,
                    valueSuffix: axisDef.unit.labelSuffix
                };
                chartOptions.series.push( seriesOptions );
            }, this);
        }, this);

        chartOptions = meso.Util.applyDefaults(this.options.highstockChartOptions, chartOptions);

        return chartOptions;
    };
    // highcharts value formatter
    AbstractHighstockChart.prototype._suffixFormatter = function(suffix) {
        return function() {
            return this.value + suffix;
        };
    };
    /**
     * Subclass may override to tweak the Highcharts series data grouping option (units
     * in particular). Defaults to matching the approximation to the field agg 
     * definition.
     */
    AbstractHighstockChart.prototype._buildSeriesDataGroupingOption = function(fieldDef) {
        return {
            units: null, // will use highcarts defaults
            approximation: AGG_TO_GROUP_APPROXIMATION[fieldDef.agg]
        };
    };

    var AGG_TO_GROUP_APPROXIMATION = (function() {
        var map = {};
        map[meso.Agg.avg] = 'average';
        map[meso.Agg.min] = 'low';
        map[meso.Agg.max] = 'high';
        map[meso.Agg.sum] = 'sum';
        return map;
    })();

    var DATA_GROUPING_DISABLED = {
        enabled : false
    };

    return AbstractHighstockChart;
})();
